@@ -1,4 +1,3 @@
-#!/usr/bin/env python2
 #
 # Copyright 2009 Facebook
 #
@@ -16,7 +15,7 @@
 
 """The Tornado web server and tools."""
 
-from __future__ import absolute_import, division, print_function, with_statement
+from __future__ import absolute_import, division, print_function
 
 # version is a human-readable version number.
 
@@ -25,5 +24,5 @@ from __future__ import absolute_import, division, print_function, with_statement
 # is zero for an official release, positive for a development branch,
 # or negative for a release candidate or beta (after the base version
 # number has been incremented)
-version = "4.1"
-version_info = (4, 1, 0, 0)
+version = "5.1.1"
+version_info = (5, 1, 1, 0)
